/* guile-irrlicht --- GNU Guile bindings for Irrlicht Engine

   Copyright (C) 2020 Javier Sancho <jsf@jsancho.org>

   This file is part of guile-irrlicht.

   guile-irrlicht is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 3 of the
   License, or (at your option) any later version.

   guile-irrlicht is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with guile-irrlicht. If not, see
   <http://www.gnu.org/licenses/>.
*/

#include <irrlicht/irrlicht.h>
#include <libguile.h>
#include "gsubr.h"
#include "gui-toolbar.h"
#include "wrapped.h"

using namespace irr;

SCM
IGUIToolBar_addButton (SCM gui_toolbar,
                       SCM id,
                       SCM text,
                       SCM tooltiptext,
                       SCM img,
                       SCM pressedimg,
                       SCM is_push_button,
                       SCM use_alpha_channel)
{
  gui::IGUIToolBar* toolbar = (gui::IGUIToolBar*) scm_to_irr_pointer (gui_toolbar);
  wchar_t* wtext = (wchar_t*) scm_to_utf32_string (text);
  wchar_t* wtooltiptext = (wchar_t*) scm_to_utf32_string (tooltiptext);

  gui::IGUIButton* button =
    toolbar->addButton (scm_to_int32 (id),
                        wtext,
                        wtooltiptext,
                        (video::ITexture*) scm_to_irr_pointer (img),
                        (video::ITexture*) scm_to_irr_pointer (pressedimg),
                        scm_to_bool (is_push_button),
                        scm_to_bool (use_alpha_channel));

  free (wtext);
  free (wtooltiptext);
  return scm_from_irr_pointer ("<gui-button>", (void*) button);
}

void
init_gui_toolbar (void)
{
  DEFINE_GSUBR ("IGUIToolBar_addButton", 8, 0, 0, IGUIToolBar_addButton);
}
